var http = require('http');
var fs = require('fs');
var path = require('path');
var url = require('url');

http.createServer(function (request, response) {

	response.setHeader("Access-Control-Allow-Origin", "*");
  response.setHeader("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
	response.setHeader("Access-Control-Allow-Methods", "POST, GET, DELETE, OPTIONS");
	
	var request_url = url.parse(request.url),
		filePath = '../' + request_url.pathname.replace('..', '.');

	if (request.method === "GET") {

		if (request_url.pathname == '/') {
			readdir('../', function(err, files) {
				files = files.map(function(f) {
					return f.replace('..\\', '').replace(/\\/g, '/');
				}).filter(function(f) {
					return !/^bin|^unins/.test(f);
				});
				response.writeHead(200, { 'Content-Type': 'application/json' });
				response.end(JSON.stringify(files), 'utf-8');
			});
		} else {
			fs.stat(filePath, function(error, stats) {
				if (error) {
					response.writeHead(404, {'Content-Type': 'text/html'});
					response.write('<!doctype html><html><head><title>404</title></head><body>404: Resource Not Found</body></html>');
					response.end();
				} else if (stats.isFile())
					fs.readFile(filePath, function(error, content) {
						if (error) {
							response.writeHead(404, {'Content-Type': 'text/html'});
							response.write('<!doctype html><html><head><title>404</title></head><body>404: Resource Not Found</body></html>');
							response.end();
						} else {
							var extname = path.extname(filePath);
							var contentType = 'text/html';
							switch (extname) {
								case '.json':
								case '.metadata':
									contentType = 'application/json';
									break;
								case '.png':
									contentType = 'image/png';
									break;      
								case '.jpg':
									contentType = 'image/jpg';
									break;
								case '.wav':
									contentType = 'audio/wav';
									break;
							}
							response.writeHead(200, { 'Content-Type': contentType });
							response.end(content, 'utf-8');
						}
					});
				else
					fs.readdir(filePath, function(error, files) {
						response.writeHead(200, { 'Content-Type': 'application/json' });
						response.end(JSON.stringify(files), 'utf-8');
					});
			});
		}
	} else if (request.method === "POST") {
		var body = '';
        request.on('data', function (data) {
            body += data;
        });
        request.on('end', function () {
			var base64Data = body.split(',')[1];
			var buffer = new Buffer(base64Data, 'base64');
			mkdirP(path.dirname(filePath), function() {
				fs.writeFileSync(filePath, buffer);
				console.log('saved ' + filePath);
				response.writeHead(200, {'Content-Type': 'text/html'});
				response.end('ok');
			});
        });
  } else if (request.method === "DELETE") {
    console.log('delete ' + filePath);
    fs.unlink(filePath, function(error) {
      if (!error) {
        response.writeHead(200, {'Content-Type': 'text/html'});
        response.end('ok');
      } else {
        response.writeHead(500, {'Content-Type': 'text/html'});
        response.end(error);
      }
    })
	} else if (request.method === "OPTIONS") {
    response.writeHead(200, {'Content-Type': 'text/html'});
    response.end('');
  }


}).listen(22549);

console.log('Server running at http://127.0.0.1:22549/');

function readdir(file_path, ignores, callback) {
  if (typeof ignores == 'function') {
    callback = ignores
    ignores  = []
  }
  var list = []

  fs.readdir(file_path, function (err, files) {
    if (err) {
      return callback(err)
    }

    var pending = files.length
    if (!pending) {
      // we are done, woop woop
      return callback(null, list)
    }

    files.forEach(function (file) {
      fs.lstat(path.join(file_path, file), function (err, stats) {
        if (err) {
          return callback(err)
        }

        file = path.join(file_path, file)
        if (stats.isDirectory()) {
          files = readdir(file, ignores, function (err, res) {
            if (err) {
              return callback(err)
            }

            list = list.concat(res)
            pending -= 1
            if (!pending) {
              callback(null, list)
            }
          })
        }
        else {
          for (var i = 0; i < ignores.length; i++) {
            if (ignores[i].test(file)) {
              pending -= 1
              if (pending <= 0) {
                callback(null, list)
              }
              return
            }
          }
          list.push(file)
          pending -= 1
          if (!pending) {
            callback(null, list)
          }
        }
      })
    })
  })
}

var _0777 = parseInt('0777', 8);

function mkdirP (p, opts, f, made) {
    if (typeof opts === 'function') {
        f = opts;
        opts = {};
    }
    else if (!opts || typeof opts !== 'object') {
        opts = { mode: opts };
    }
    
    var mode = opts.mode;
    var xfs = opts.fs || fs;
    
    if (mode === undefined) {
        mode = _0777 & (~process.umask());
    }
    if (!made) made = null;
    
    var cb = f || function () {};
    p = path.resolve(p);
    
    xfs.mkdir(p, mode, function (er) {
        if (!er) {
            made = made || p;
            return cb(null, made);
        }
        switch (er.code) {
            case 'ENOENT':
                mkdirP(path.dirname(p), opts, function (er, made) {
                    if (er) cb(er, made);
                    else mkdirP(p, opts, cb, made);
                });
                break;

            // In the case of any other error, just see if there's a dir
            // there already.  If so, then hooray!  If not, then something
            // is borked.
            default:
                xfs.stat(p, function (er2, stat) {
                    // if the stat fails, then that's super weird.
                    // let the original error be the failure reason.
                    if (er2 || !stat.isDirectory()) cb(er, made)
                    else cb(null, made);
                });
                break;
        }
    });
}